@@ -65,46 +65,47 @@ typedef struct grib_accessor_concept
 extern grib_accessor_class* grib_accessor_class_gen;
 
 static grib_accessor_class _grib_accessor_class_concept = {
-    &grib_accessor_class_gen,      /* super                     */
-    "concept",                     /* name                      */
-    sizeof(grib_accessor_concept), /* size                      */
-    0,                             /* inited */
-    &init_class,                   /* init_class */
-    &init,                         /* init                      */
-    0,                             /* post_init                      */
-    &destroy,                      /* free mem                       */
-    &dump,                         /* describes himself         */
-    0,                             /* get length of section     */
-    &string_length,                /* get length of string      */
-    &value_count,                  /* get number of values      */
-    0,                             /* get number of bytes      */
-    0,                             /* get offset to bytes           */
-    &get_native_type,              /* get native type               */
-    0,                             /* get sub_section                */
-    0,                             /* grib_pack procedures long      */
-    0,                             /* grib_pack procedures long      */
-    &pack_long,                    /* grib_pack procedures long      */
-    &unpack_long,                  /* grib_unpack procedures long    */
-    &pack_double,                  /* grib_pack procedures double    */
-    &unpack_double,                /* grib_unpack procedures double  */
-    &pack_string,                  /* grib_pack procedures string    */
-    &unpack_string,                /* grib_unpack procedures string  */
-    0,                             /* grib_pack array procedures string    */
-    0,                             /* grib_unpack array procedures string  */
-    0,                             /* grib_pack procedures bytes     */
-    0,                             /* grib_unpack procedures bytes   */
-    0,                             /* pack_expression */
-    0,                             /* notify_change   */
-    0,                             /* update_size   */
-    0,                             /* preferred_size   */
-    0,                             /* resize   */
-    0,                             /* nearest_smaller_value */
-    0,                             /* next accessor    */
-    &compare,                      /* compare vs. another accessor   */
-    0,                             /* unpack only ith value          */
-    0,                             /* unpack a subarray         */
-    0,                             /* clear          */
-    0,                             /* clone accessor          */
+    &grib_accessor_class_gen,                      /* super */
+    "concept",                      /* name */
+    sizeof(grib_accessor_concept),  /* size */
+    0,                           /* inited */
+    &init_class,                 /* init_class */
+    &init,                       /* init */
+    0,                  /* post_init */
+    &destroy,                    /* free mem */
+    &dump,                       /* describes himself */
+    0,                /* get length of section */
+    &string_length,              /* get length of string */
+    &value_count,                /* get number of values */
+    0,                 /* get number of bytes */
+    0,                /* get offset to bytes */
+    &get_native_type,            /* get native type */
+    0,                /* get sub_section */
+    0,               /* grib_pack procedures long */
+    0,                 /* grib_pack procedures long */
+    &pack_long,                  /* grib_pack procedures long */
+    &unpack_long,                /* grib_unpack procedures long */
+    &pack_double,                /* grib_pack procedures double */
+    &unpack_double,              /* grib_unpack procedures double */
+    &pack_string,                /* grib_pack procedures string */
+    &unpack_string,              /* grib_unpack procedures string */
+    0,          /* grib_pack array procedures string */
+    0,        /* grib_unpack array procedures string */
+    0,                 /* grib_pack procedures bytes */
+    0,               /* grib_unpack procedures bytes */
+    0,            /* pack_expression */
+    0,              /* notify_change */
+    0,                /* update_size */
+    0,             /* preferred_size */
+    0,                     /* resize */
+    0,      /* nearest_smaller_value */
+    0,                       /* next accessor */
+    &compare,                    /* compare vs. another accessor */
+    0,      /* unpack only ith value */
+    0,  /* unpack a given set of elements */
+    0,     /* unpack a subarray */
+    0,                      /* clear */
+    0,                 /* clone accessor */
 };
 
 
@@ -113,33 +114,37 @@ grib_accessor_class* grib_accessor_class_concept = &_grib_accessor_class_concept
 
 static void init_class(grib_accessor_class* c)
 {
-    c->next_offset            = (*(c->super))->next_offset;
-    c->byte_count             = (*(c->super))->byte_count;
-    c->byte_offset            = (*(c->super))->byte_offset;
-    c->sub_section            = (*(c->super))->sub_section;
-    c->pack_missing           = (*(c->super))->pack_missing;
-    c->is_missing             = (*(c->super))->is_missing;
-    c->pack_string_array      = (*(c->super))->pack_string_array;
-    c->unpack_string_array    = (*(c->super))->unpack_string_array;
-    c->pack_bytes             = (*(c->super))->pack_bytes;
-    c->unpack_bytes           = (*(c->super))->unpack_bytes;
-    c->pack_expression        = (*(c->super))->pack_expression;
-    c->notify_change          = (*(c->super))->notify_change;
-    c->update_size            = (*(c->super))->update_size;
-    c->preferred_size         = (*(c->super))->preferred_size;
-    c->resize                 = (*(c->super))->resize;
-    c->nearest_smaller_value  = (*(c->super))->nearest_smaller_value;
-    c->next                   = (*(c->super))->next;
-    c->unpack_double_element  = (*(c->super))->unpack_double_element;
-    c->unpack_double_subarray = (*(c->super))->unpack_double_subarray;
-    c->clear                  = (*(c->super))->clear;
-    c->make_clone             = (*(c->super))->make_clone;
+    c->next_offset    =    (*(c->super))->next_offset;
+    c->byte_count    =    (*(c->super))->byte_count;
+    c->byte_offset    =    (*(c->super))->byte_offset;
+    c->sub_section    =    (*(c->super))->sub_section;
+    c->pack_missing    =    (*(c->super))->pack_missing;
+    c->is_missing    =    (*(c->super))->is_missing;
+    c->pack_string_array    =    (*(c->super))->pack_string_array;
+    c->unpack_string_array    =    (*(c->super))->unpack_string_array;
+    c->pack_bytes    =    (*(c->super))->pack_bytes;
+    c->unpack_bytes    =    (*(c->super))->unpack_bytes;
+    c->pack_expression    =    (*(c->super))->pack_expression;
+    c->notify_change    =    (*(c->super))->notify_change;
+    c->update_size    =    (*(c->super))->update_size;
+    c->preferred_size    =    (*(c->super))->preferred_size;
+    c->resize    =    (*(c->super))->resize;
+    c->nearest_smaller_value    =    (*(c->super))->nearest_smaller_value;
+    c->next    =    (*(c->super))->next;
+    c->unpack_double_element    =    (*(c->super))->unpack_double_element;
+    c->unpack_double_element_set    =    (*(c->super))->unpack_double_element_set;
+    c->unpack_double_subarray    =    (*(c->super))->unpack_double_subarray;
+    c->clear    =    (*(c->super))->clear;
+    c->make_clone    =    (*(c->super))->make_clone;
 }
 
 /* END_CLASS_IMP */
 
 #define MAX_CONCEPT_STRING_LENGTH 255
 
+#define FALSE 0
+#define TRUE  1
+
 /* Note: A fast cut-down version of strcmp which does NOT return -1 */
 /* 0 means input strings are equal and 1 means not equal */
 GRIB_INLINE static int grib_inline_strcmp(const char* a, const char* b)
@@ -163,11 +168,12 @@ static void dump(grib_accessor* a, grib_dumper* dumper)
     grib_dump_string(dumper, a, NULL);
 }
 
+/* Return 1 (=True) or 0 (=False) */
 static int concept_condition_expression_true(grib_handle* h, grib_concept_condition* c)
 {
     long lval;
     long lres      = 0;
-    int ok         = 0;
+    int ok         = FALSE; /* Boolean */
     int err        = 0;
     const int type = grib_expression_native_type(h, c->expression);
 
@@ -207,34 +213,38 @@ static int concept_condition_expression_true(grib_handle* h, grib_concept_condit
     return ok;
 }
 
+/* Return 1 (=True) or 0 (=False) */
 static int concept_condition_iarray_true(grib_handle* h, grib_concept_condition* c)
 {
-    long* val;
+    long* val = NULL;
     size_t size = 0, i;
-    int ret;
+    int ret; /* Boolean */
     int err = 0;
 
     err = grib_get_size(h, c->name, &size);
-    if (err == 0 || size != grib_iarray_used_size(c->iarray))
-        return 0;
+    if (err || size != grib_iarray_used_size(c->iarray))
+        return FALSE;
 
     val = (long*)grib_context_malloc_clear(h->context, sizeof(long) * size);
 
     err = grib_get_long_array(h, c->name, val, &size);
-    if (err == 0)
-        return 0;
-
-    ret = 1;
+    if (err) {
+        grib_context_free(h->context, val);
+        return FALSE;
+    }
+    ret = TRUE;
     for (i = 0; i < size; i++) {
         if (val[i] != c->iarray->v[i]) {
-            ret = 0;
+            ret = FALSE;
             break;
         }
     }
 
+    grib_context_free(h->context, val);
     return ret;
 }
 
+/* Return 1 (=True) or 0 (=False) */
 static int concept_condition_true(grib_handle* h, grib_concept_condition* c)
 {
     if (c->expression == NULL)
@@ -357,18 +367,23 @@ static int grib_concept_apply(grib_accessor* a, const char* name)
         if (err) {
             size_t i = 0, concept_count = 0;
             long dummy = 0, editionNumber = 0;
-            char* all_concept_vals[MAX_NUM_CONCEPT_VALUES] = {
-                NULL,
-            }; /* sorted array containing concept values */
+            char centre_s[32] = {0,};
+            size_t centre_len = sizeof(centre_s);
+            char* all_concept_vals[MAX_NUM_CONCEPT_VALUES] = {NULL,}; /* sorted array containing concept values */
             grib_concept_value* pCon = concepts;
 
             grib_context_log(h->context, GRIB_LOG_ERROR, "concept: no match for %s=%s", act->name, name);
+            if (grib_get_long(h, "edition", &editionNumber) == GRIB_SUCCESS &&
+                grib_get_string(h, "centre", centre_s, &centre_len) == GRIB_SUCCESS) {
+                grib_context_log(h->context, GRIB_LOG_ERROR, "concept: input handle edition=%ld, centre=%s", editionNumber, centre_s);
+            }
             if (strcmp(act->name, "paramId") == 0 && string_to_long(name, &dummy) == GRIB_SUCCESS) {
                 grib_context_log(h->context, GRIB_LOG_ERROR,
                                  "Please check the Parameter Database 'https://apps.ecmwf.int/codes/grib/param-db/?id=%s'", name);
             }
-            if (grib_get_long(h, "edition", &editionNumber) == GRIB_SUCCESS) {
-                grib_context_log(h->context, GRIB_LOG_ERROR, "concept: input handle edition=%ld", editionNumber);
+            if (strcmp(act->name, "shortName") == 0) {
+                grib_context_log(h->context, GRIB_LOG_ERROR,
+                                 "Please check the Parameter Database 'https://apps.ecmwf.int/codes/grib/param-db/'");
             }
 
             /* Create a list of all possible values for this concept and sort it */
@@ -522,9 +537,10 @@ static int unpack_long(grib_accessor* a, long* val, size_t* len)
 
     *val = atol(p);
     *len = 1;
-#if 0
+#ifdef DEBUG
     /* ECC-980: Changes reverted because of side-effects!
-     * e.g. marsType being a codetable and concept! see ifsParam
+     * e.g. marsType being a codetable and concept! see ifsParam.
+     * Keep this check in DEBUG mode only
      */
     {
         char *endptr;
@@ -626,7 +642,9 @@ static int unpack_string(grib_accessor* a, char* val, size_t* len)
 
     slen = strlen(p) + 1;
     if (*len < slen) {
-        grib_context_log(a->context, GRIB_LOG_ERROR, "Variable unpack_string. Wrong size for %s, it is %d bytes big (len=%d)", a->name, slen, *len);
+        grib_context_log(a->context, GRIB_LOG_ERROR,
+                        "Concept unpack_string. Wrong size for %s, value='%s' which requires %lu bytes (len=%lu)",
+                         a->name, p, slen, *len);
         *len = slen;
         return GRIB_BUFFER_TOO_SMALL;
     }
