@@ -17,7 +17,7 @@
    CLASS      = accessor
    SUPER      = grib_accessor_class_gen
    IMPLEMENTS = init
-   IMPLEMENTS = unpack_double;unpack_double_element
+   IMPLEMENTS = unpack_double;unpack_double_element;unpack_double_element_set
    IMPLEMENTS = pack_double
    IMPLEMENTS = value_count
    IMPLEMENTS = dump;get_native_type
@@ -48,63 +48,65 @@ static void dump(grib_accessor*, grib_dumper*);
 static void init(grib_accessor*, const long, grib_arguments*);
 static void init_class(grib_accessor_class*);
 static int unpack_double_element(grib_accessor*, size_t i, double* val);
+static int unpack_double_element_set(grib_accessor*, const size_t* index_array, size_t len, double* val_array);
 
 typedef struct grib_accessor_data_apply_bitmap
 {
     grib_accessor att;
     /* Members defined in gen */
     /* Members defined in data_apply_bitmap */
-    const char* coded_values;
-    const char* bitmap;
-    const char* missing_value;
-    const char* number_of_data_points;
-    const char* number_of_values;
-    const char* binary_scale_factor;
+    const char*  coded_values;
+    const char*  bitmap;
+    const char*  missing_value;
+    const char*  number_of_data_points;
+    const char*  number_of_values;
+    const char*  binary_scale_factor;
 } grib_accessor_data_apply_bitmap;
 
 extern grib_accessor_class* grib_accessor_class_gen;
 
 static grib_accessor_class _grib_accessor_class_data_apply_bitmap = {
-    &grib_accessor_class_gen,                /* super                     */
-    "data_apply_bitmap",                     /* name                      */
-    sizeof(grib_accessor_data_apply_bitmap), /* size                      */
-    0,                                       /* inited */
-    &init_class,                             /* init_class */
-    &init,                                   /* init                      */
-    0,                                       /* post_init                      */
-    0,                                       /* free mem                       */
-    &dump,                                   /* describes himself         */
-    0,                                       /* get length of section     */
-    0,                                       /* get length of string      */
-    &value_count,                            /* get number of values      */
-    0,                                       /* get number of bytes      */
-    0,                                       /* get offset to bytes           */
-    &get_native_type,                        /* get native type               */
-    0,                                       /* get sub_section                */
-    0,                                       /* grib_pack procedures long      */
-    0,                                       /* grib_pack procedures long      */
-    0,                                       /* grib_pack procedures long      */
-    0,                                       /* grib_unpack procedures long    */
-    &pack_double,                            /* grib_pack procedures double    */
-    &unpack_double,                          /* grib_unpack procedures double  */
-    0,                                       /* grib_pack procedures string    */
-    0,                                       /* grib_unpack procedures string  */
-    0,                                       /* grib_pack array procedures string    */
-    0,                                       /* grib_unpack array procedures string  */
-    0,                                       /* grib_pack procedures bytes     */
-    0,                                       /* grib_unpack procedures bytes   */
-    0,                                       /* pack_expression */
-    0,                                       /* notify_change   */
-    0,                                       /* update_size   */
-    0,                                       /* preferred_size   */
-    0,                                       /* resize   */
-    0,                                       /* nearest_smaller_value */
-    0,                                       /* next accessor    */
-    0,                                       /* compare vs. another accessor   */
-    &unpack_double_element,                  /* unpack only ith value          */
-    0,                                       /* unpack a subarray         */
-    0,                                       /* clear          */
-    0,                                       /* clone accessor          */
+    &grib_accessor_class_gen,                      /* super */
+    "data_apply_bitmap",                      /* name */
+    sizeof(grib_accessor_data_apply_bitmap),  /* size */
+    0,                           /* inited */
+    &init_class,                 /* init_class */
+    &init,                       /* init */
+    0,                  /* post_init */
+    0,                    /* free mem */
+    &dump,                       /* describes himself */
+    0,                /* get length of section */
+    0,              /* get length of string */
+    &value_count,                /* get number of values */
+    0,                 /* get number of bytes */
+    0,                /* get offset to bytes */
+    &get_native_type,            /* get native type */
+    0,                /* get sub_section */
+    0,               /* grib_pack procedures long */
+    0,                 /* grib_pack procedures long */
+    0,                  /* grib_pack procedures long */
+    0,                /* grib_unpack procedures long */
+    &pack_double,                /* grib_pack procedures double */
+    &unpack_double,              /* grib_unpack procedures double */
+    0,                /* grib_pack procedures string */
+    0,              /* grib_unpack procedures string */
+    0,          /* grib_pack array procedures string */
+    0,        /* grib_unpack array procedures string */
+    0,                 /* grib_pack procedures bytes */
+    0,               /* grib_unpack procedures bytes */
+    0,            /* pack_expression */
+    0,              /* notify_change */
+    0,                /* update_size */
+    0,             /* preferred_size */
+    0,                     /* resize */
+    0,      /* nearest_smaller_value */
+    0,                       /* next accessor */
+    0,                    /* compare vs. another accessor */
+    &unpack_double_element,      /* unpack only ith value */
+    &unpack_double_element_set,  /* unpack a given set of elements */
+    0,     /* unpack a subarray */
+    0,                      /* clear */
+    0,                 /* clone accessor */
 };
 
 
@@ -113,32 +115,32 @@ grib_accessor_class* grib_accessor_class_data_apply_bitmap = &_grib_accessor_cla
 
 static void init_class(grib_accessor_class* c)
 {
-    c->next_offset            = (*(c->super))->next_offset;
-    c->string_length          = (*(c->super))->string_length;
-    c->byte_count             = (*(c->super))->byte_count;
-    c->byte_offset            = (*(c->super))->byte_offset;
-    c->sub_section            = (*(c->super))->sub_section;
-    c->pack_missing           = (*(c->super))->pack_missing;
-    c->is_missing             = (*(c->super))->is_missing;
-    c->pack_long              = (*(c->super))->pack_long;
-    c->unpack_long            = (*(c->super))->unpack_long;
-    c->pack_string            = (*(c->super))->pack_string;
-    c->unpack_string          = (*(c->super))->unpack_string;
-    c->pack_string_array      = (*(c->super))->pack_string_array;
-    c->unpack_string_array    = (*(c->super))->unpack_string_array;
-    c->pack_bytes             = (*(c->super))->pack_bytes;
-    c->unpack_bytes           = (*(c->super))->unpack_bytes;
-    c->pack_expression        = (*(c->super))->pack_expression;
-    c->notify_change          = (*(c->super))->notify_change;
-    c->update_size            = (*(c->super))->update_size;
-    c->preferred_size         = (*(c->super))->preferred_size;
-    c->resize                 = (*(c->super))->resize;
-    c->nearest_smaller_value  = (*(c->super))->nearest_smaller_value;
-    c->next                   = (*(c->super))->next;
-    c->compare                = (*(c->super))->compare;
-    c->unpack_double_subarray = (*(c->super))->unpack_double_subarray;
-    c->clear                  = (*(c->super))->clear;
-    c->make_clone             = (*(c->super))->make_clone;
+    c->next_offset    =    (*(c->super))->next_offset;
+    c->string_length    =    (*(c->super))->string_length;
+    c->byte_count    =    (*(c->super))->byte_count;
+    c->byte_offset    =    (*(c->super))->byte_offset;
+    c->sub_section    =    (*(c->super))->sub_section;
+    c->pack_missing    =    (*(c->super))->pack_missing;
+    c->is_missing    =    (*(c->super))->is_missing;
+    c->pack_long    =    (*(c->super))->pack_long;
+    c->unpack_long    =    (*(c->super))->unpack_long;
+    c->pack_string    =    (*(c->super))->pack_string;
+    c->unpack_string    =    (*(c->super))->unpack_string;
+    c->pack_string_array    =    (*(c->super))->pack_string_array;
+    c->unpack_string_array    =    (*(c->super))->unpack_string_array;
+    c->pack_bytes    =    (*(c->super))->pack_bytes;
+    c->unpack_bytes    =    (*(c->super))->unpack_bytes;
+    c->pack_expression    =    (*(c->super))->pack_expression;
+    c->notify_change    =    (*(c->super))->notify_change;
+    c->update_size    =    (*(c->super))->update_size;
+    c->preferred_size    =    (*(c->super))->preferred_size;
+    c->resize    =    (*(c->super))->resize;
+    c->nearest_smaller_value    =    (*(c->super))->nearest_smaller_value;
+    c->next    =    (*(c->super))->next;
+    c->compare    =    (*(c->super))->compare;
+    c->unpack_double_subarray    =    (*(c->super))->unpack_double_subarray;
+    c->clear    =    (*(c->super))->clear;
+    c->make_clone    =    (*(c->super))->make_clone;
 }
 
 /* END_CLASS_IMP */
@@ -261,6 +263,7 @@ static int unpack_double(grib_accessor* a, double* val, size_t* len)
 static int unpack_double_element(grib_accessor* a, size_t idx, double* val)
 {
     grib_accessor_data_apply_bitmap* self = (grib_accessor_data_apply_bitmap*)a;
+    grib_handle* gh      = grib_handle_of_accessor(a);
     int err = 0, i = 0;
     size_t cidx          = 0;
     double missing_value = 0;
@@ -273,13 +276,13 @@ static int unpack_double_element(grib_accessor* a, size_t idx, double* val)
     if (err)
         return err;
 
-    if (!grib_find_accessor(grib_handle_of_accessor(a), self->bitmap))
-        return grib_get_double_element_internal(grib_handle_of_accessor(a), self->coded_values, idx, val);
+    if (!grib_find_accessor(gh, self->bitmap))
+        return grib_get_double_element_internal(gh, self->coded_values, idx, val);
 
-    if ((err = grib_get_double_internal(grib_handle_of_accessor(a), self->missing_value, &missing_value)) != GRIB_SUCCESS)
+    if ((err = grib_get_double_internal(gh, self->missing_value, &missing_value)) != GRIB_SUCCESS)
         return err;
 
-    if ((err = grib_get_double_element_internal(grib_handle_of_accessor(a), self->bitmap, idx, val)) != GRIB_SUCCESS)
+    if ((err = grib_get_double_element_internal(gh, self->bitmap, idx, val)) != GRIB_SUCCESS)
         return err;
 
     if (*val == 0) {
@@ -291,7 +294,7 @@ static int unpack_double_element(grib_accessor* a, size_t idx, double* val)
     if (bvals == NULL)
         return GRIB_OUT_OF_MEMORY;
 
-    if ((err = grib_get_double_array_internal(grib_handle_of_accessor(a), self->bitmap, bvals, &n_vals)) != GRIB_SUCCESS)
+    if ((err = grib_get_double_array_internal(gh, self->bitmap, bvals, &n_vals)) != GRIB_SUCCESS)
         return err;
 
     cidx = 0;
@@ -301,7 +304,87 @@ static int unpack_double_element(grib_accessor* a, size_t idx, double* val)
 
     grib_context_free(a->context, bvals);
 
-    return grib_get_double_element_internal(grib_handle_of_accessor(a), self->coded_values, cidx, val);
+    return grib_get_double_element_internal(gh, self->coded_values, cidx, val);
+}
+
+static int unpack_double_element_set(grib_accessor* a, const size_t* index_array, size_t len, double* val_array)
+{
+    grib_accessor_data_apply_bitmap* self = (grib_accessor_data_apply_bitmap*)a;
+    grib_handle* gh = grib_handle_of_accessor(a);
+    int err = 0, all_missing = 1;
+    size_t cidx        = 0; /* index into the coded_values array */
+    size_t* cidx_array = NULL; /* array of indexes into the coded_values */
+    double* cval_array = NULL; /* array of values of the coded_values */
+    double missing_value = 0;
+    double* bvals        = NULL;
+    size_t n_vals = 0, i = 0, j = 0, idx = 0, count_1s = 0, ci = 0;
+    long nn = 0;
+
+    err    = grib_value_count(a, &nn);
+    n_vals = nn;
+    if (err) return err;
+
+    if (!grib_find_accessor(gh, self->bitmap))
+        return grib_get_double_element_set_internal(gh, self->coded_values, index_array, len, val_array);
+
+    if ((err = grib_get_double_internal(gh, self->missing_value, &missing_value)) != GRIB_SUCCESS)
+        return err;
+
+    err = grib_get_double_element_set_internal(gh, self->bitmap, index_array, len, val_array);
+    if (err) return err;
+    for (i = 0; i < len; i++) {
+        if (val_array[i] == 0) {
+            val_array[i] = missing_value;
+        } else {
+            all_missing = 0;
+            count_1s++;
+        }
+    }
+
+    if (all_missing) {
+        return GRIB_SUCCESS;
+    }
+
+    /* At this point val_array contains entries which are either missing_value or 1 */
+    /* Now we need to dig into the codes values with index array of count_1s */
+
+    bvals = (double*)grib_context_malloc(a->context, n_vals * sizeof(double));
+    if (!bvals) return GRIB_OUT_OF_MEMORY;
+
+    if ((err = grib_get_double_array_internal(gh, self->bitmap, bvals, &n_vals)) != GRIB_SUCCESS)
+        return err;
+
+    cidx_array = (size_t*)grib_context_malloc(a->context, count_1s * sizeof(size_t));
+    cval_array = (double*)grib_context_malloc(a->context, count_1s * sizeof(double));
+    
+    ci = 0;
+    for (i = 0; i < len; i++) {
+        if (val_array[i] == 1) {
+            idx = index_array[i];
+            cidx = 0;
+            for (j = 0; j < idx; j++) {
+                cidx += bvals[j];
+            }
+            Assert(ci < count_1s);
+            cidx_array[ci++] = cidx;
+        }
+    }
+    err = grib_get_double_element_set_internal(gh, self->coded_values, cidx_array, count_1s, cval_array);
+    if (err) return err;
+
+    /* Transfer from cval_array to our result val_array */
+    ci = 0;
+    for (i = 0; i < len; i++) {
+        if (val_array[i] == 1) {
+            val_array[i] = cval_array[ci++];
+        }
+    }
+
+    grib_context_free(a->context, bvals);
+    grib_context_free(a->context, cidx_array);
+    grib_context_free(a->context, cval_array);
+
+    return GRIB_SUCCESS;
 }
 
 static int pack_double(grib_accessor* a, const double* val, size_t* len)
@@ -324,7 +407,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t* len)
         /*printf("SETTING TOTAL number_of_data_points %s %ld\n",self->number_of_data_points,*len);*/
         if (self->number_of_data_points)
             grib_set_long_internal(hand, self->number_of_data_points, *len);
-        
+
         err = grib_set_double_array_internal(hand, self->coded_values, val, *len);
         return err;
     }
