@@ -187,7 +187,7 @@ struct grib_values
 */
 typedef struct grib_handle grib_handle;
 
-/*! Grib multi field handle,   structure used to build multi field GRIB messages.
+/*! Grib multi field handle,   structure used to build multi-field GRIB messages.
     \ingroup grib_handle
  */
 typedef struct grib_multi_handle grib_multi_handle;
@@ -235,12 +235,12 @@ typedef struct bufr_descriptor bufr_descriptor;
 typedef struct bufr_descriptors_array bufr_descriptors_array;
 typedef struct bufr_descriptors_map_list bufr_descriptors_map_list;
 
-grib_fieldset* grib_fieldset_new_from_files(grib_context* c, char* filenames[], int nfiles, char** keys, int nkeys, const char* where_string, const char* order_by_string, int* err);
+grib_fieldset* grib_fieldset_new_from_files(grib_context* c, const char* filenames[], int nfiles, const char** keys, int nkeys, const char* where_string, const char* order_by_string, int* err);
 void grib_fieldset_delete(grib_fieldset* set);
 void grib_fieldset_rewind(grib_fieldset* set);
 int grib_fieldset_apply_order_by(grib_fieldset* set, const char* order_by_string);
 grib_handle* grib_fieldset_next_handle(grib_fieldset* set, int* err);
-int grib_fieldset_count(grib_fieldset* set);
+int grib_fieldset_count(const grib_fieldset* set);
 int grib_values_check(grib_handle* h, grib_values* values, int count);
 
 /*! \defgroup grib_index The grib_index
@@ -253,7 +253,7 @@ The grib_index is the structure giving indexed access to messages in a file.
 typedef struct grib_index grib_index;
 
 /**
- *  Create a new index form a file. The file is indexed with the keys in argument.
+ *  Create a new index from a file. The file is indexed with the keys in argument.
  *
  * @param c           : context  (NULL for default context)
  * @param filename    : name of the file of messages to be indexed
@@ -387,7 +387,7 @@ int grib_index_select_string(grib_index* index, const char* key, const char* val
  *
  * @param index       : an index created from a file.
  * @param err         : 0 if OK, integer value on error. GRIB_END_OF_INDEX when no more handles are contained in the index.
- * @return            grib handle.
+ * @return            GRIB handle.
  */
 grib_handle* grib_handle_new_from_index(grib_index* index, int* err);
 
@@ -474,7 +474,7 @@ grib_handle* grib_handle_new_from_message(grib_context* c, const void* data, siz
 /**
 *  Create a handle from a user message in memory. The message will not be freed at the end.
 *  The message will be copied as soon as a modification is needed.
-*  This function works also with multi field messages.
+*  This function works also with multi-field messages.
 *  Note: The data pointer argument may be modified
 *
 * @param c           : the context from which the handle will be created (NULL for default context)
@@ -502,7 +502,7 @@ grib_handle* grib_handle_new_from_message_copy(grib_context* c, const void* data
  *  The message is copied at the creation of the handle
  *
  * @param c           : the context from which the handle will be created (NULL for default context)
- * @param sample_name : the name of the sample file (without the .tmpl extension)
+ * @param sample_name : the name of the GRIB sample file
  * @return            the new handle, NULL if the resource is invalid or a problem is encountered
  */
 grib_handle* grib_handle_new_from_samples(grib_context* c, const char* sample_name);
@@ -526,7 +526,7 @@ grib_handle* grib_handle_clone(const grib_handle* h);
 int grib_handle_delete(grib_handle* h);
 
 /**
- *  Create an empty multi field handle.
+ *  Create an empty multi-field handle.
  *  Remember always to delete the multi handle when it is not needed anymore to avoid
  *  memory leaks.
  *
@@ -536,7 +536,7 @@ grib_multi_handle* grib_multi_handle_new(grib_context* c);
 
 /**
  *  Append the sections starting with start_section of the message pointed by h at
- *  the end of the multi field handle mh.
+ *  the end of the multi-field handle mh.
  *  Remember always to delete the multi handle when it is not needed anymore to avoid
  *  memory leaks.
  *
@@ -548,20 +548,20 @@ grib_multi_handle* grib_multi_handle_new(grib_context* c);
 int grib_multi_handle_append(grib_handle* h, int start_section, grib_multi_handle* mh);
 
 /**
- * Delete multi field handle.
+ * Delete multi-field handle.
  *
- * @param mh           : The multi field handle to be deleted.
+ * @param mh          : The multi-field handle to be deleted.
  * @return            0 if OK, integer value on error
  */
 int grib_multi_handle_delete(grib_multi_handle* mh);
 
 /**
- *  Write a multi field handle in a file.
- *  Remember always to delete the multi handle when it is not needed anymore to avoid
+ *  Write a multi-field handle in a file.
+ *  Remember to delete the multi handle when it is not needed anymore to avoid
  *  memory leaks.
  *
- * @param mh           : The multi field handle to be written.
- * @param f            : File on which the file handle is written.
+ * @param mh          : The multi field handle to be written.
+ * @param f           : File on which the file handle is written.
  * @return            0 if OK, integer value on error
  */
 int grib_multi_handle_write(grib_multi_handle* mh, FILE* f);
@@ -805,16 +805,16 @@ int grib_get_double(const grib_handle* h, const char* key, double* value);
 int grib_get_double_element(const grib_handle* h, const char* key, int i, double* value);
 
 /**
-*  Get as double array the elements of the "key" array whose indexes are listed in the input array i
+*  Get as double array the elements of the "key" array whose indexes are listed in the input array "index_array"
 *
 * @param h           : the handle to get the data from
 * @param key         : the key to be searched
-* @param i           : zero-based array of indexes
-* @param size        : size of the i and value arrays
+* @param index_array : zero-based array of indexes
+* @param size        : size of the index_array and value arrays
 * @param value       : the double array for the data values
 * @return            0 if OK, integer value on error
 */
-int grib_get_double_elements(const grib_handle* h, const char* key, int* i, long size, double* value);
+int grib_get_double_elements(const grib_handle* h, const char* key, const int* index_array, long size, double* value);
 
 /**
 *  Get a string value from a key, if several keys of the same name are present, the last one is returned
@@ -1221,21 +1221,21 @@ void grib_context_set_print_proc(grib_context* c, grib_print_proc printp);
 void grib_context_set_logging_proc(grib_context* c, grib_log_proc logp);
 
 /**
-*  Turn on support for multiple fields in single grib messages
+*  Turn on support for multi-fields in single GRIB messages
 *
 * @param c            : the context to be modified
 */
 void grib_multi_support_on(grib_context* c);
 
 /**
-*  Turn off support for multiple fields in single GRIB messages
+*  Turn off support for multi-fields in single GRIB messages
 *
 * @param c            : the context to be modified
 */
 void grib_multi_support_off(grib_context* c);
 
 /**
-*  Reset file handle in multiple field support mode
+*  Reset file handle in multi-field support mode
 *
 * @param c            : the context to be modified
 * @param f            : the file pointer
@@ -1399,10 +1399,6 @@ struct grib_points
     size_t size;
 };
 
-grib_box* grib_box_new(grib_handle* h, int* error);
-grib_points* grib_box_get_points(grib_box* box, double north, double west, double south, double east, int* err);
-int grib_points_get_values(grib_handle* h, grib_points* points, double* val);
-
 
 /* --------------------------------------- */
 #define GRIB_UTIL_GRID_SPEC_REGULAR_LL 1
@@ -1423,7 +1419,7 @@ int grib_points_get_values(grib_handle* h, grib_points* points, double* val);
 
 typedef struct grib_util_grid_spec
 {
-    int grid_type;
+    int grid_type; /* e.g. GRIB_UTIL_GRID_SPEC_REGULAR_LL etc */
 
     /* Grid */
     long Ni;
@@ -1450,18 +1446,18 @@ typedef struct grib_util_grid_spec
     /* Gaussian number */
     long N;
 
-    /* bitmap */
+    /* Bitmap */
     long bitmapPresent;
-    double missingValue;
+    double missingValue; /* 0 means use the default */
 
-    /* pl list for reduced */
+    /* 'pl' array for reduced Gaussian grids */
     const long* pl;
     long pl_size;
 
     /* Spherical harmonics */
     long truncation;
 
-    /* polar stereographic */
+    /* Polar stereographic */
     double orientationOfTheGridInDegrees;
     long DyInMetres;
     long DxInMetres;
@@ -1470,7 +1466,7 @@ typedef struct grib_util_grid_spec
 
 typedef struct grib_util_grid_spec2
 {
-    int grid_type;
+    int grid_type;  /* e.g. GRIB_UTIL_GRID_SPEC_REGULAR_LL etc */
     const char* grid_name; /* e.g. N320 */
 
     /* Grid */
@@ -1499,18 +1495,18 @@ typedef struct grib_util_grid_spec2
     /* Gaussian number */
     long N;
 
-    /* bitmap */
+    /* Bitmap */
     long bitmapPresent;
-    double missingValue;
+    double missingValue;  /* 0 means use the default */
 
-    /* pl list for reduced */
+    /* 'pl' array for reduced Gaussian grids */
     const long* pl;
     long pl_size;
 
     /* Spherical harmonics */
     long truncation;
 
-    /* polar stereographic */
+    /* Polar stereographic */
     double orientationOfTheGridInDegrees;
     long DyInMetres;
     long DxInMetres;
