@@ -118,9 +118,11 @@ double rint(double x);
 #include <strings.h>
 #endif
 
+/*
 #if GRIB_LINUX_PTHREADS
 extern int pthread_mutexattr_settype(pthread_mutexattr_t* attr, int type);
 #endif
+*/
 
 #if GRIB_PTHREADS
 #include <pthread.h>
@@ -308,7 +310,10 @@ typedef void (*accessor_destroy_proc)(grib_context*, grib_accessor*);
 
 typedef int (*accessor_unpack_long_proc)(grib_accessor*, long*, size_t* len);
 typedef int (*accessor_unpack_double_proc)(grib_accessor*, double*, size_t* len);
+
 typedef int (*accessor_unpack_double_element_proc)(grib_accessor*, size_t, double*);
+typedef int (*accessor_unpack_double_element_set_proc)(grib_accessor*, const size_t*, size_t, double*);
+
 typedef int (*accessor_unpack_double_subarray_proc)(grib_accessor*, double*, size_t, size_t);
 typedef int (*accessor_unpack_string_proc)(grib_accessor*, char*, size_t* len);
 typedef int (*accessor_unpack_string_array_proc)(grib_accessor*, char**, size_t* len);
@@ -588,7 +593,7 @@ struct grib_section
 struct grib_iterator_class
 {
     grib_iterator_class** super;
-    char* name;
+    const char* name;
     size_t size;
 
     int inited;
@@ -606,7 +611,7 @@ struct grib_iterator_class
 struct grib_nearest_class
 {
     grib_nearest_class** super;
-    char* name;
+    const char* name;
     size_t size;
 
     int inited;
@@ -621,7 +626,7 @@ struct grib_nearest_class
 struct grib_box_class
 {
     grib_box_class** super;
-    char* name;
+    const char* name;
     size_t size;
     int inited;
     box_init_class_proc init_class;
@@ -658,7 +663,7 @@ struct grib_dumper
 struct grib_dumper_class
 {
     grib_dumper_class** super;
-    char* name;
+    const char* name;
     size_t size;
     int inited;
     dumper_init_class_proc init_class;
@@ -972,6 +977,7 @@ struct grib_accessor_class
     accessor_next_proc next;
     accessor_compare_proc compare;
     accessor_unpack_double_element_proc unpack_double_element;
+    accessor_unpack_double_element_set_proc unpack_double_element_set;
     accessor_unpack_double_subarray_proc unpack_double_subarray;
     accessor_clear_proc clear;
     accessor_clone_proc make_clone;
@@ -1164,7 +1170,6 @@ struct grib_arguments
 {
     struct grib_arguments* next;
     grib_expression* expression;
-    char value[80];
 };
 
 
@@ -1589,7 +1594,7 @@ typedef struct j2k_encode_helper
 
 } j2k_encode_helper;
 
-#include "grib_api_prototypes.h"
+#include "eccodes_prototypes.h"
 
 
 #ifdef __cplusplus
