@@ -42,24 +42,23 @@ or edit "dumper.class" and rerun ./make_class.pl
 
 */
 
-static void init_class(grib_dumper_class*);
-static int init(grib_dumper* d);
-static int destroy(grib_dumper*);
-static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_bits(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_double(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_bytes(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_values(grib_dumper* d, grib_accessor* a);
-static void dump_label(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_section(grib_dumper* d, grib_accessor* a, grib_block_of_accessors* block);
-static void header(grib_dumper*, grib_handle*);
-static void footer(grib_dumper*, grib_handle*);
+static void init_class      (grib_dumper_class*);
+static int init            (grib_dumper* d);
+static int destroy         (grib_dumper*);
+static void dump_long       (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_bits       (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_double     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_string     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_string_array     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_bytes      (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_values     (grib_dumper* d, grib_accessor* a);
+static void dump_label      (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_section    (grib_dumper* d, grib_accessor* a,grib_block_of_accessors* block);
+static void header         (grib_dumper*,grib_handle*);
+static void footer         (grib_dumper*,grib_handle*);
 
-typedef struct grib_dumper_bufr_encode_fortran
-{
-    grib_dumper dumper;
+typedef struct grib_dumper_bufr_encode_fortran {
+    grib_dumper          dumper;  
     /* Members defined in bufr_encode_fortran */
     long section_offset;
     long empty;
@@ -71,24 +70,24 @@ typedef struct grib_dumper_bufr_encode_fortran
 
 
 static grib_dumper_class _grib_dumper_class_bufr_encode_fortran = {
-    0,                                       /* super                     */
-    "bufr_encode_fortran",                   /* name                      */
-    sizeof(grib_dumper_bufr_encode_fortran), /* size                      */
-    0,                                       /* inited */
-    &init_class,                             /* init_class */
-    &init,                                   /* init                      */
-    &destroy,                                /* free mem                       */
-    &dump_long,                              /* dump long         */
-    &dump_double,                            /* dump double    */
-    &dump_string,                            /* dump string    */
-    &dump_string_array,                      /* dump string array   */
-    &dump_label,                             /* dump labels  */
-    &dump_bytes,                             /* dump bytes  */
-    &dump_bits,                              /* dump bits   */
-    &dump_section,                           /* dump section      */
-    &dump_values,                            /* dump values   */
-    &header,                                 /* header   */
-    &footer,                                 /* footer   */
+    0,                              /* super                     */
+    "bufr_encode_fortran",                              /* name                      */
+    sizeof(grib_dumper_bufr_encode_fortran),     /* size                      */
+    0,                                   /* inited */
+    &init_class,                         /* init_class */
+    &init,                               /* init                      */
+    &destroy,                            /* free mem                       */
+    &dump_long,                          /* dump long         */
+    &dump_double,                        /* dump double    */
+    &dump_string,                        /* dump string    */
+    &dump_string_array,                        /* dump string array   */
+    &dump_label,                         /* dump labels  */
+    &dump_bytes,                         /* dump bytes  */
+    &dump_bits,                          /* dump bits   */
+    &dump_section,                       /* dump section      */
+    &dump_values,                        /* dump values   */
+    &header,                             /* header   */
+    &footer,                             /* footer   */
 };
 
 grib_dumper_class* grib_dumper_class_bufr_encode_fortran = &_grib_dumper_class_bufr_encode_fortran;
@@ -109,14 +108,6 @@ GRIB_INLINE static int grib_inline_strcmp(const char* a, const char* b)
     return (*a == 0 && *b == 0) ? 0 : 1;
 }
 
-typedef struct string_count string_count;
-struct string_count
-{
-    char* value;
-    int count;
-    string_count* next;
-};
-
 static int depth = 0;
 
 static void init_class(grib_dumper_class* c) {}
@@ -124,7 +115,7 @@ static void init_class(grib_dumper_class* c) {}
 static int init(grib_dumper* d)
 {
     grib_dumper_bufr_encode_fortran* self = (grib_dumper_bufr_encode_fortran*)d;
-    grib_context* c                       = d->handle->context;
+    grib_context* c                       = d->context;
     self->section_offset                  = 0;
     self->empty                           = 1;
     d->count                              = 1;
@@ -140,7 +131,7 @@ static int destroy(grib_dumper* d)
     grib_dumper_bufr_encode_fortran* self = (grib_dumper_bufr_encode_fortran*)d;
     grib_string_list* next                = self->keys;
     grib_string_list* cur                 = NULL;
-    grib_context* c                       = d->handle->context;
+    grib_context* c                       = d->context;
     while (next) {
         cur  = next;
         next = next->next;
@@ -393,6 +384,11 @@ static void dump_values_attribute(grib_dumper* d, grib_accessor* a, const char*
     (void)err; /* TODO */
 }
 
+static int is_hidden(grib_accessor* a)
+{
+    return ( (a->flags & GRIB_ACCESSOR_FLAG_HIDDEN) != 0 );
+}
+
 static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
 {
     grib_dumper_bufr_encode_fortran* self = (grib_dumper_bufr_encode_fortran*)d;
@@ -408,8 +404,12 @@ static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
     grib_handle* h                  = grib_handle_of_accessor(a);
     int doing_unexpandedDescriptors = 0;
 
-    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0)
-        return;
+    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0) { /* key does not have the dump attribute */
+        int skip = 1;
+        /* See ECC-1107 */
+        if (!is_hidden(a) && strcmp(a->name, "messageLength") == 0) skip = 0;
+        if (skip) return;
+    }
 
     doing_unexpandedDescriptors = (strcmp(a->name, "unexpandedDescriptors") == 0);
     grib_value_count(a, &count);
@@ -695,15 +695,13 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
     fprintf(self->dumper.out, "    \"%s\" /)\n", values[size - 1]);
 
     if (self->isLeaf == 0) {
+        char* prefix;
+        int dofree = 0;
+
         if ((r = compute_bufr_key_rank(h, self->keys, a->name)) != 0)
             fprintf(self->dumper.out, "  call codes_set_string_array(ibufr,'#%d#%s',svalues)\n", r, a->name);
         else
             fprintf(self->dumper.out, "  call codes_set_string_array(ibufr,'%s',svalues)\n", a->name);
-    }
-
-    if (self->isLeaf == 0) {
-        char* prefix;
-        int dofree = 0;
 
         if (r != 0) {
             prefix = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name) + 10));
@@ -718,7 +716,7 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
             grib_context_free(c, prefix);
         depth -= 2;
     }
-
+    for (i=0; i<size; ++i) grib_context_free(c, values[i]);
     grib_context_free(c, values);
     (void)err; /* TODO */
 }
@@ -739,18 +737,8 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
     if (size == 0)
         return;
 
-    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0) {
-        /* ECC-356: Solution for the special local section key 'keyMore' and its alias 'ident' */
-        int skip = 1;
-        if ((a->flags & GRIB_ACCESSOR_FLAG_HIDDEN) != 0) {
-            if (strcmp(a->name, "keyMore") == 0 && grib_is_defined(h, "ls.ident")) {
-                skip     = 0;
-                acc_name = "ident";
-            }
-        }
-        if (skip)
-            return;
-    }
+    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
+        return;
 
     value = (char*)grib_context_malloc_clear(c, size);
     if (!value) {
@@ -769,7 +757,7 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
 
     while (*p) {
         if (!isprint(*p))
-            *p = '.';
+            *p = '?';
         p++;
     }
 
@@ -949,7 +937,7 @@ static void header(grib_dumper* d, grib_handle* h)
     }
     fprintf(self->dumper.out, "  call codes_bufr_new_from_samples(ibufr,'%s',iret)\n", sampleName);
     fprintf(self->dumper.out, "  if (iret/=CODES_SUCCESS) then\n");
-    fprintf(self->dumper.out, "    print *,'ERROR creating BUFR from %s'\n", sampleName);
+    fprintf(self->dumper.out, "    print *,'ERROR: Failed to create BUFR from %s'\n", sampleName);
     fprintf(self->dumper.out, "    stop 1\n");
     fprintf(self->dumper.out, "  endif\n");
 }
