@@ -41,22 +41,21 @@ or edit "dumper.class" and rerun ./make_class.pl
 
 */
 
-static void init_class(grib_dumper_class*);
-static int init(grib_dumper* d);
-static int destroy(grib_dumper*);
-static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_bits(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_double(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_bytes(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_values(grib_dumper* d, grib_accessor* a);
-static void dump_label(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_section(grib_dumper* d, grib_accessor* a, grib_block_of_accessors* block);
+static void init_class      (grib_dumper_class*);
+static int init            (grib_dumper* d);
+static int destroy         (grib_dumper*);
+static void dump_long       (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_bits       (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_double     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_string     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_string_array     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_bytes      (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_values     (grib_dumper* d, grib_accessor* a);
+static void dump_label      (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_section    (grib_dumper* d, grib_accessor* a,grib_block_of_accessors* block);
 
-typedef struct grib_dumper_json
-{
-    grib_dumper dumper;
+typedef struct grib_dumper_json {
+    grib_dumper          dumper;  
     /* Members defined in json */
     long section_offset;
     long begin;
@@ -68,24 +67,24 @@ typedef struct grib_dumper_json
 
 
 static grib_dumper_class _grib_dumper_class_json = {
-    0,                        /* super                     */
-    "json",                   /* name                      */
-    sizeof(grib_dumper_json), /* size                      */
-    0,                        /* inited */
-    &init_class,              /* init_class */
-    &init,                    /* init                      */
-    &destroy,                 /* free mem                       */
-    &dump_long,               /* dump long         */
-    &dump_double,             /* dump double    */
-    &dump_string,             /* dump string    */
-    &dump_string_array,       /* dump string array   */
-    &dump_label,              /* dump labels  */
-    &dump_bytes,              /* dump bytes  */
-    &dump_bits,               /* dump bits   */
-    &dump_section,            /* dump section      */
-    &dump_values,             /* dump values   */
-    0,                        /* header   */
-    0,                        /* footer   */
+    0,                              /* super                     */
+    "json",                              /* name                      */
+    sizeof(grib_dumper_json),     /* size                      */
+    0,                                   /* inited */
+    &init_class,                         /* init_class */
+    &init,                               /* init                      */
+    &destroy,                            /* free mem                       */
+    &dump_long,                          /* dump long         */
+    &dump_double,                        /* dump double    */
+    &dump_string,                        /* dump string    */
+    &dump_string_array,                        /* dump string array   */
+    &dump_label,                         /* dump labels  */
+    &dump_bytes,                         /* dump bytes  */
+    &dump_bits,                          /* dump bits   */
+    &dump_section,                       /* dump section      */
+    &dump_values,                        /* dump values   */
+    0,                             /* header   */
+    0,                             /* footer   */
 };
 
 grib_dumper_class* grib_dumper_class_json = &_grib_dumper_class_json;
@@ -381,6 +380,7 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
     size_t size = 0, i = 0;
     grib_context* c = NULL;
     int err         = 0;
+    int is_missing  = 0;
     long count      = 0;
     c               = a->context;
 
@@ -423,9 +423,13 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
     fprintf(self->dumper.out, "\n%-*s[", depth, " ");
     depth += 2;
     for (i = 0; i < size - 1; i++) {
-        fprintf(self->dumper.out, "%-*s\"%s\",\n", depth, " ", values[i]);
+        is_missing = grib_is_missing_string(a, (unsigned char*)values[i], strlen(values[i]));
+        if (is_missing) fprintf(self->dumper.out, "%-*s%s,\n", depth, " ", "null");
+        else            fprintf(self->dumper.out, "%-*s\"%s\",\n", depth, " ", values[i]);
     }
-    fprintf(self->dumper.out, "%-*s\"%s\"\n", depth, " ", values[i]);
+    is_missing = grib_is_missing_string(a, (unsigned char*)values[i], strlen(values[i]));
+    if (is_missing) fprintf(self->dumper.out, "%-*s%s", depth, " ", "null");
+    else            fprintf(self->dumper.out, "%-*s\"%s\"", depth, " ", values[i]);
 
     depth -= 2;
     fprintf(self->dumper.out, "\n%-*s]", depth, " ");
@@ -450,24 +454,14 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
 {
     grib_dumper_json* self      = (grib_dumper_json*)d;
     char value[MAX_STRING_SIZE] = {0,}; /* See ECC-710 */
+    size_t size                 = MAX_STRING_SIZE;
     char* p              = NULL;
-    size_t size          = MAX_STRING_SIZE;
     int is_missing       = 0;
     int err              = 0;
     const char* acc_name = a->name;
 
     if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0) {
-        /* ECC-356: Solution for the special local section key 'keyMore' and its alias 'ident' */
-        int skip = 1;
-        if ((a->flags & GRIB_ACCESSOR_FLAG_HIDDEN) != 0) {
-            grib_handle* h = grib_handle_of_accessor(a);
-            if (strcmp(a->name, "keyMore") == 0 && grib_is_defined(h, "ls.ident")) {
-                skip     = 0;
-                acc_name = "ident";
-            }
-        }
-        if (skip)
-            return;
+        return;
     }
 
     /* ECC-710: It is MUCH slower determining the string length here
@@ -489,7 +483,12 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
     self->empty = 0;
 
     err = grib_unpack_string(a, value, &size);
-    Assert(size < MAX_STRING_SIZE);
+    if (err) {
+        sprintf(value, " *** ERR=%d (%s) [dump_string on '%s']",
+                err, grib_get_error_message(err), a->name);
+    } else {
+        Assert(size < MAX_STRING_SIZE);
+    }
     p = value;
     if (grib_is_missing_string(a, (unsigned char*)value, size)) {
         is_missing = 1;
@@ -497,7 +496,9 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
 
     while (*p) {
         if (!isprint(*p))
-            *p = '.';
+            *p = '?';
+        if (*p == '"')
+            *p = '\''; /* ECC-1401 */
         p++;
     }
 
