@@ -23,9 +23,9 @@
    MEMBERS    = double* lons
    MEMBERS    = int  lons_count
    MEMBERS    = double* distances
-   MEMBERS    = int* k
-   MEMBERS    = int* i
-   MEMBERS    = int* j
+   MEMBERS    = size_t* k
+   MEMBERS    = size_t* i
+   MEMBERS    = size_t* j
    MEMBERS    = const char* Ni
    MEMBERS    = const char* Nj
    END_CLASS_DEF
@@ -43,28 +43,27 @@ or edit "nearest.class" and rerun ./make_class.pl
 */
 
 
-static void init_class(grib_nearest_class*);
+static void init_class              (grib_nearest_class*);
 
-static int init(grib_nearest* nearest, grib_handle* h, grib_arguments* args);
-static int find(grib_nearest* nearest, grib_handle* h, double inlat, double inlon, unsigned long flags, double* outlats, double* outlons, double* values, double* distances, int* indexes, size_t* len);
-static int destroy(grib_nearest* nearest);
+static int init               (grib_nearest* nearest,grib_handle* h,grib_arguments* args);
+static int find(grib_nearest* nearest, grib_handle* h,double inlat, double inlon, unsigned long flags, double* outlats,double* outlons, double *values,double *distances, int *indexes,size_t *len);
+static int destroy            (grib_nearest* nearest);
 
-typedef struct grib_nearest_regular
-{
-    grib_nearest nearest;
+typedef struct grib_nearest_regular{
+  grib_nearest nearest;
     /* Members defined in gen */
     const char* values_key;
     const char* radius;
     int cargs;
     /* Members defined in regular */
     double* lats;
-    int lats_count;
+    int  lats_count;
     double* lons;
-    int lons_count;
+    int  lons_count;
     double* distances;
-    int* k;
-    int* i;
-    int* j;
+    size_t* k;
+    size_t* i;
+    size_t* j;
     const char* Ni;
     const char* Nj;
 } grib_nearest_regular;
@@ -72,14 +71,14 @@ typedef struct grib_nearest_regular
 extern grib_nearest_class* grib_nearest_class_gen;
 
 static grib_nearest_class _grib_nearest_class_regular = {
-    &grib_nearest_class_gen,      /* super                     */
-    "regular",                    /* name                      */
-    sizeof(grib_nearest_regular), /* size of instance          */
-    0,                            /* inited */
-    &init_class,                  /* init_class */
-    &init,                        /* constructor               */
-    &destroy,                     /* destructor                */
-    &find,                        /* find nearest              */
+    &grib_nearest_class_gen,                         /* super */
+    "regular",                         /* name */
+    sizeof(grib_nearest_regular),      /* size of instance */
+    0,                              /* inited */
+    &init_class,                    /* init_class */
+    &init,                          /* constructor */
+    &destroy,                       /* destructor */
+    &find,                          /* find nearest */
 };
 
 grib_nearest_class* grib_nearest_class_regular = &_grib_nearest_class_regular;
@@ -90,13 +89,16 @@ static void init_class(grib_nearest_class* c)
 }
 /* END_CLASS_IMP */
 
+
+#define NUM_NEIGHBOURS 4
+
 static int init(grib_nearest* nearest, grib_handle* h, grib_arguments* args)
 {
     grib_nearest_regular* self = (grib_nearest_regular*)nearest;
     self->Ni                   = grib_arguments_get_name(h, args, self->cargs++);
     self->Nj                   = grib_arguments_get_name(h, args, self->cargs++);
-    self->i                    = (int*)grib_context_malloc(h->context, 2 * sizeof(int));
-    self->j                    = (int*)grib_context_malloc(h->context, 2 * sizeof(int));
+    self->i                    = (size_t*)grib_context_malloc(h->context, 2 * sizeof(size_t));
+    self->j                    = (size_t*)grib_context_malloc(h->context, 2 * sizeof(size_t));
     return 0;
 }
 
@@ -108,13 +110,10 @@ static int find(grib_nearest* nearest, grib_handle* h,
     grib_nearest_regular* self = (grib_nearest_regular*) nearest;
     int ret=0,kk=0,ii=0,jj=0;
     size_t nvalues=0;
+    double radiusInKm;
 
-    long iradius;
-    double radius;
-
-    if( (ret =  grib_get_long(h,self->radius,&iradius))!= GRIB_SUCCESS)
+    if ((ret = grib_nearest_get_radius(h, &radiusInKm)) != GRIB_SUCCESS)
         return ret;
-    radius=((double)iradius)/1000.0;
 
     if (!nearest->h || (flags & GRIB_NEAREST_SAME_DATA)==0 || nearest->h!=h) {
         grib_iterator* iter=NULL;
@@ -175,7 +174,6 @@ static int find(grib_nearest* nearest, grib_handle* h,
 
         }
         nearest->h=h;
-
     }
 
     if (!self->distances || (flags & GRIB_NEAREST_SAME_POINT)==0
@@ -186,14 +184,14 @@ static int find(grib_nearest* nearest, grib_handle* h,
         grib_binary_search(self->lons,self->lons_count-1,inlon,
                 &(self->i[0]),&(self->i[1]));
         if (!self->distances)
-            self->distances=(double*)grib_context_malloc( nearest->context,4*sizeof(double));
+            self->distances=(double*)grib_context_malloc( nearest->context,NUM_NEIGHBOURS*sizeof(double));
         if (!self->k)
-            self->k=(int*)grib_context_malloc( nearest->context,4*sizeof(int));
+            self->k=(int*)grib_context_malloc( nearest->context,NUM_NEIGHBOURS*sizeof(int));
         kk=0;
         for (ii=0;ii<2;ii++) {
             for (jj=0;jj<2;jj++) {
                 self->k[kk]=self->i[ii]+self->lons_count*self->j[jj]-1;
-                self->distances[kk]=grib_nearest_distance(radius,inlon,inlat,
+                self->distances[kk]=geographic_distance_spherical(radius,inlon,inlat,
                         self->lons[self->i[ii]],self->lats[self->j[jj]]);
                 kk++;
             }
@@ -218,7 +216,7 @@ static int find(grib_nearest* nearest, grib_handle* h,
 static int is_rotated_grid(grib_handle* h)
 {
     long is_rotated = 0;
-    int err         = grib_get_long(h, "is_rotated_grid", &is_rotated);
+    int err         = grib_get_long(h, "isRotatedGrid", &is_rotated);
     if (!err && is_rotated)
         return 1;
     return 0;
@@ -232,8 +230,7 @@ static int find(grib_nearest* nearest, grib_handle* h,
     grib_nearest_regular* self = (grib_nearest_regular*)nearest;
     int ret = 0, kk = 0, ii = 0, jj = 0;
     size_t nvalues = 0;
-    long iradius;
-    double radius;
+    double radiusInKm;
 
     grib_iterator* iter = NULL;
     double lat = 0, lon = 0;
@@ -249,15 +246,12 @@ static int find(grib_nearest* nearest, grib_handle* h,
         return ret;
     nearest->values_count = nvalues;
 
-    if (grib_is_missing(h, self->radius, &ret)) {
-        grib_context_log(h->context, GRIB_LOG_DEBUG, "Key '%s' is missing", self->radius);
-        return ret ? ret : GRIB_GEOCALCULUS_PROBLEM;
-    }
-
-    if ((ret = grib_get_long(h, self->radius, &iradius)) != GRIB_SUCCESS)
+    if ((ret = grib_nearest_get_radius(h, &radiusInKm)) != GRIB_SUCCESS)
         return ret;
-    radius = ((double)iradius) / 1000.0;
 
+    /* Compute lat/lon info, create iterator etc if it's the 1st time or different grid.
+     * This is for performance: if the grid has not changed, we only do this once
+     * and reuse for other messages */
     if (!nearest->h || (flags & GRIB_NEAREST_SAME_GRID) == 0) {
         double dummy = 0;
         double olat = 1.e10, olon = 1.e10;
@@ -328,8 +322,8 @@ static int find(grib_nearest* nearest, grib_handle* h,
             return ret;
         }
         while (grib_iterator_next(iter, &lat, &lon, &dummy)) {
-            if (olat != lat) {
-                Assert(ilat < self->lats_count);
+            if (ilat < self->lats_count && olat != lat) {
+                /* Assert(ilat < self->lats_count); */
                 self->lats[ilat++] = lat;
                 olat               = lat;
             }
@@ -342,6 +336,9 @@ static int find(grib_nearest* nearest, grib_handle* h,
     }
     nearest->h = h;
 
+    /* Compute distances if it's the 1st time or different point or different grid.
+     * This is for performance: if the grid and the input point have not changed
+     * we only do this once and reuse for other messages */
     if (!self->distances || (flags & GRIB_NEAREST_SAME_POINT) == 0 || (flags & GRIB_NEAREST_SAME_GRID) == 0) {
         int nearest_lons_found = 0;
 
@@ -402,14 +399,14 @@ static int find(grib_nearest* nearest, grib_handle* h,
                                &(self->i[0]), &(self->i[1]));
 
         if (!self->distances)
-            self->distances = (double*)grib_context_malloc(nearest->context, 4 * sizeof(double));
+            self->distances = (double*)grib_context_malloc(nearest->context, NUM_NEIGHBOURS * sizeof(double));
         if (!self->k)
-            self->k = (int*)grib_context_malloc(nearest->context, 4 * sizeof(int));
+            self->k = (size_t*)grib_context_malloc(nearest->context, NUM_NEIGHBOURS * sizeof(size_t));
         kk = 0;
         for (jj = 0; jj < 2; jj++) {
             for (ii = 0; ii < 2; ii++) {
                 self->k[kk]         = self->i[ii] + self->lons_count * self->j[jj];
-                self->distances[kk] = grib_nearest_distance(radius, inlon, inlat,
+                self->distances[kk] = geographic_distance_spherical(radiusInKm, inlon, inlat,
                                                             self->lons[self->i[ii]], self->lats[self->j[jj]]);
                 kk++;
             }
@@ -430,6 +427,13 @@ static int find(grib_nearest* nearest, grib_handle* h,
      *   if (ret) return ret;
      */
 
+    if (values) {
+        /* See ECC-1403 and ECC-499 */
+        /* Performance: Decode the field once and get all 4 values */
+        if ((ret = grib_get_double_element_set(h, self->values_key, self->k, NUM_NEIGHBOURS, values)) != GRIB_SUCCESS)
+            return ret;
+    }
+
     for (jj = 0; jj < 2; jj++) {
         for (ii = 0; ii < 2; ii++) {
             distances[kk] = self->distances[kk];
@@ -442,9 +446,11 @@ static int find(grib_nearest* nearest, grib_handle* h,
                 outlats[kk] = new_lat;
                 outlons[kk] = new_lon;
             }
-            if (values) { /* ECC-499 */
-                grib_get_double_element_internal(h, self->values_key, self->k[kk], &(values[kk]));
-            }
+            /* See ECC-1403 and ECC-499
+             * if (values) {
+             *   grib_get_double_element_internal(h, self->values_key, self->k[kk], &(values[kk]));
+             *}
+             */
             /* Using the brute force approach described above */
             /* Assert(self->k[kk] < nvalues); */
             /* values[kk]=nearest->values[self->k[kk]]; */
