@@ -46,7 +46,7 @@ const char* tool_description =
 const char* tool_name  = "grib_compare";
 const char* tool_usage = "[options] grib_file1 grib_file2";
 
-typedef double (*compare_double_proc)(double*, double*, double);
+typedef double (*compare_double_proc)(const double*, const double*, double);
 static int compare_handles(grib_handle* h1, grib_handle* h2, grib_runtime_options* options);
 
 typedef struct grib_error grib_error;
@@ -82,7 +82,8 @@ static int onlyListed          = 1;
 static int headerMode          = 0;
 static int morein1             = 0;
 static int morein2             = 0;
-static int listFromCommandLine;
+static int listFromCommandLine = 0;
+static int editionIndependent  = 0;
 static int verbose             = 0;
 static double tolerance_factor = 1;
 
@@ -91,6 +92,8 @@ static int global_counter   = 0;
 static int theStart         = -1;
 static int theEnd           = -1;
 
+#define MINIMUM(x, y) ((x) < (y) ? (x) : (y))
+
 GRIB_INLINE static int grib_inline_strcmp(const char* a, const char* b)
 {
     if (*a != *b)
@@ -122,7 +125,7 @@ GRIB_INLINE static int grib_inline_rstrcmp(const char* a, const char* b)
 }
 
 /* Returns 0 when the values are considered the same */
-static double compare_double_absolute(double* a, double* b, double tolerance)
+static double compare_double_absolute(const double* a, const double* b, double tolerance)
 {
     double ret = 0;
     double d   = fabs(*a - *b);
@@ -134,7 +137,7 @@ static double compare_double_absolute(double* a, double* b, double tolerance)
 }
 
 /* Returns 0 when the values are considered the same */
-static double compare_double_relative(double* a, double* b, double tolerance)
+static double compare_double_relative(const double* a, const double* b, double tolerance)
 {
     double relativeError;
 
@@ -186,7 +189,7 @@ int grib_tool_before_getopt(grib_runtime_options* options)
 
 int grib_tool_init(grib_runtime_options* options)
 {
-    int ret = 0, i = 0;
+    int ret = 0;
     int nfiles            = 1;
     const char orderby[]  = "md5Headers";
     grib_context* context = grib_context_get_default();
@@ -214,6 +217,9 @@ int grib_tool_init(grib_runtime_options* options)
     if (grib_options_on("c:") || grib_options_on("e"))
         listFromCommandLine = 1;
 
+    if (grib_options_on("e"))
+        editionIndependent = 1;
+
     if (grib_options_on("a"))
         onlyListed = 0;
     else
@@ -234,6 +240,7 @@ int grib_tool_init(grib_runtime_options* options)
     }
 
     if (grib_options_on("b:")) {
+        int i;
         grib_string_list* next = 0;
         blocklist              = (grib_string_list*)grib_context_malloc_clear(context, sizeof(grib_string_list));
         blocklist->value       = grib_context_strdup(context, options->set_values[0].name);
@@ -246,8 +253,11 @@ int grib_tool_init(grib_runtime_options* options)
         context->blocklist = blocklist;
     }
 
+    /* Check 1st file is not a directory */
+    exit_if_input_is_directory(tool_name, options->infile_extra->name);
+
     if (grib_options_on("r")) {
-        char* filename[1];
+        const char* filename[1];
         filename[0]      = options->infile_extra->name; /* First file */
         options->random  = 1;
         options->orderby = strdup(orderby);
@@ -272,6 +282,7 @@ int grib_tool_init(grib_runtime_options* options)
     global_tolerance = 0;
     compare_double   = &compare_double_absolute;
     if (grib_options_on("R:")) {
+        int i;
         global_tolerance = 0;
         for (i = 0; i < options->tolerance_count; i++) {
             if (!strcmp((options->tolerance[i]).name, "all")) {
@@ -524,8 +535,7 @@ int grib_tool_finalise_action(grib_runtime_options* options)
 
     while ((handle1 = grib_handle_new_from_file(c, options->infile_extra->file, &err))) {
         morein1++;
-        if (handle1)
-            grib_handle_delete(handle1);
+        grib_handle_delete(handle1);
     }
 
     error += morein1 + morein2;
@@ -1071,17 +1081,18 @@ static int compare_values(grib_runtime_options* options, grib_handle* h1, grib_h
             }
 
             if (err1 == GRIB_SUCCESS && err2 == GRIB_SUCCESS) {
-                if (memcmp(uval1, uval2, len1) != 0) {
-                    for (i = 0; i < len1; i++)
+                const size_t len_min = MINIMUM(len1, len2);
+                if (memcmp(uval1, uval2, len_min) != 0) {
+                    for (i = 0; i < len_min; i++)
                         if (uval1[i] != uval2[i]) {
                             printInfo(h1);
                             save_error(c, name);
-                            if (len1 == 1)
+                            if (len_min == 1)
                                 printf("[%s] byte values are different: [%02x] and [%02x]\n",
                                        name, uval1[i], uval2[i]);
                             else
                                 printf("[%s] byte value %d of %ld is different: [%02x] and [%02x]\n",
-                                       name, i, (long)len1, uval1[i], uval2[i]);
+                                       name, i, (long)len_min, uval1[i], uval2[i]);
 
                             err1 = GRIB_VALUE_MISMATCH;
                             break;
@@ -1175,14 +1186,15 @@ static int compare_handles(grib_handle* h1, grib_handle* h2, grib_runtime_option
             if (blocklisted(options->compare[i].name))
                 continue;
             if (options->compare[i].type == GRIB_NAMESPACE) {
+                int num_keys_in_namespace = 0;
                 iter = grib_keys_iterator_new(h1, 0, options->compare[i].name);
                 if (!iter) {
-                    printf("ERROR: unable to get keys iterator\n");
+                    printf("ERROR: unable to get keys iterator for namespace \"%s\".\n", options->compare[i].name);
                     exit(1);
                 }
                 while (grib_keys_iterator_next(iter)) {
                     name = grib_keys_iterator_get_name(iter);
-                    /*printf("----- comparing %s\n",name);*/
+                    num_keys_in_namespace++;
 
                     if (blocklisted(name))
                         continue;
@@ -1190,6 +1202,9 @@ static int compare_handles(grib_handle* h1, grib_handle* h2, grib_runtime_option
                         err++;
                 }
                 grib_keys_iterator_delete(iter);
+                if (num_keys_in_namespace == 0 && !editionIndependent) {
+                    printf("ERROR: namespace \"%s\" does not contain any key.\n", options->compare[i].name);
+                }
             }
             else {
                 if (compare_values(options, h1, h2, options->compare[i].name, options->compare[i].type))
