@@ -31,7 +31,9 @@ static grib_options_help grib_options_help_list[] = {
       "\n\t\tAll the keys in this list are skipped in the comparison. Bit-by-bit compare on.\n" },
     { "B:", "'order by' directive",
       "\n\t\tOrder by. The output will be ordered according to the 'order by' directive."
-      "\n\t\tExample: \"step:i asc, centre desc\" (step numeric ascending and centre descending)\n" },
+      "\n\t\tExample: \"step:i asc, centre desc\" (step numeric ascending and centre descending)"
+      "\n\t\tDefault sort mode is 'asc'\n"
+    },
     { "c:", "key[:i|d|s|n],key[:i|d|s|n],...",
       "\n\t\tOnly the listed keys or namespaces (:n) are compared. The optional letter after the colon is used "
       "\n\t\tto force the type in the comparison: i->integer, d->float, s->string, n->namespace."
@@ -103,7 +105,7 @@ static grib_options_help grib_options_help_list[] = {
     { "T:", "T | B | M | A", "Message type. T->GTS, B->BUFR, M->METAR (Experimental), A->Any (Experimental).\n\t\t\t\tThe input file is interpreted according to the message type.\n" },
     { "V", 0, "Version.\n" },
     { "W:", "width", "\n\t\tMinimum width of each column in output. Default is 10.\n" },
-    { "X:", "offset", "\n\t\tInput file offset in bytes. Processing of the input file will start from \"offset\".\n" },
+    { "X:", "offset", "\n\t\tInput file offset in bytes. Processing of the input file will start from the given offset.\n" },
     { "x", 0, "Fast parsing option, only headers are loaded.\n" },
     { "k:", "key1,key2,...",
       "\n\t\tSpecify a list of keys to index on. By default the input files are indexed on the MARS keys."
@@ -118,9 +120,9 @@ static int grib_options_help_count = sizeof(grib_options_help_list) / sizeof(gri
 void usage(void)
 {
     int i = 0;
-    printf("\nNAME \t%s\n\n", grib_tool_name);
-    printf("DESCRIPTION\n\t%s\n\n", grib_tool_description);
-    printf("USAGE \n\t%s %s\n\n", grib_tool_name, grib_tool_usage);
+    printf("\nNAME \t%s\n\n", tool_name);
+    printf("DESCRIPTION\n\t%s\n\n", tool_description);
+    printf("USAGE \n\t%s %s\n\n", tool_name, tool_usage);
     printf("OPTIONS\n");
     for (i = 0; i < grib_options_count; i++) {
         if (grib_options[i].command_line)
@@ -333,7 +335,7 @@ int grib_process_runtime_options(grib_context* context, int argc, char** argv, g
     }
 
     options->requested_print_keys_count = MAX_KEYS;
-    ret                                 = parse_keyval_string(grib_tool_name, karg, 0, GRIB_TYPE_UNDEFINED,
+    ret                                 = parse_keyval_string(tool_name, karg, 0, GRIB_TYPE_UNDEFINED,
                               options->requested_print_keys, &(options->requested_print_keys_count));
     if (ret == GRIB_INVALID_ARGUMENT)
         usage();
@@ -360,7 +362,7 @@ int grib_process_runtime_options(grib_context* context, int argc, char** argv, g
     if (grib_options_on("s:")) {
         sarg                      = grib_options_get_option("s:");
         options->set_values_count = MAX_KEYS;
-        ret                       = parse_keyval_string(grib_tool_name, sarg, 1, GRIB_TYPE_UNDEFINED, options->set_values, &(options->set_values_count));
+        ret                       = parse_keyval_string(tool_name, sarg, 1, GRIB_TYPE_UNDEFINED, options->set_values, &(options->set_values_count));
         if (ret == GRIB_INVALID_ARGUMENT)
             usage();
     }
@@ -368,7 +370,7 @@ int grib_process_runtime_options(grib_context* context, int argc, char** argv, g
     if (grib_options_on("b:")) {
         barg                      = grib_options_get_option("b:");
         options->set_values_count = MAX_KEYS;
-        ret                       = parse_keyval_string(grib_tool_name, barg, 0, GRIB_TYPE_STRING, options->set_values, &(options->set_values_count));
+        ret                       = parse_keyval_string(tool_name, barg, 0, GRIB_TYPE_STRING, options->set_values, &(options->set_values_count));
         if (ret == GRIB_INVALID_ARGUMENT)
             usage();
     }
@@ -376,7 +378,7 @@ int grib_process_runtime_options(grib_context* context, int argc, char** argv, g
     if (grib_options_on("c:")) {
         sarg                   = grib_options_get_option("c:");
         options->compare_count = MAX_KEYS;
-        ret                    = parse_keyval_string(grib_tool_name, sarg, 0, GRIB_TYPE_UNDEFINED, options->compare,
+        ret                    = parse_keyval_string(tool_name, sarg, 0, GRIB_TYPE_UNDEFINED, options->compare,
                                   &(options->compare_count));
         if (ret == GRIB_INVALID_ARGUMENT)
             usage();
@@ -392,7 +394,7 @@ int grib_process_runtime_options(grib_context* context, int argc, char** argv, g
     warg = grib_options_get_option("w:");
 
     options->constraints_count = MAX_KEYS;
-    ret                        = parse_keyval_string(grib_tool_name, warg, 1, GRIB_TYPE_UNDEFINED,
+    ret                        = parse_keyval_string(tool_name, warg, 1, GRIB_TYPE_UNDEFINED,
                               options->constraints, &(options->constraints_count));
     if (ret == GRIB_INVALID_ARGUMENT)
         usage();
@@ -400,7 +402,7 @@ int grib_process_runtime_options(grib_context* context, int argc, char** argv, g
     return GRIB_SUCCESS;
 }
 
-char* grib_options_get_help(char* id)
+const char* grib_options_get_help(const char* id)
 {
     int i      = 0;
     char msg[] = "ERROR: help not found for option ";
@@ -422,7 +424,7 @@ char* grib_options_get_help(char* id)
     return err;
 }
 
-char* grib_options_get_args(char* id)
+const char* grib_options_get_args(const char* id)
 {
     int i        = 0;
     char empty[] = "";
@@ -437,7 +439,8 @@ char* grib_options_get_args(char* id)
             if (grib_options[i].args != NULL) {
                 free(err);
                 return grib_options[i].args;
-            } else {
+            }
+            else {
                 break;
             }
         }
@@ -447,7 +450,8 @@ char* grib_options_get_args(char* id)
             if (grib_options_help_list[i].args != NULL) {
                 free(err);
                 return (char*)grib_options_help_list[i].args;
-            } else {
+            }
+            else {
                 return err;
             }
         }
@@ -458,9 +462,9 @@ char* grib_options_get_args(char* id)
 void usage_doxygen(void)
 {
     int i = 0;
-    printf("/*!  \\page %s %s\n", grib_tool_name, grib_tool_name);
-    printf("\\section DESCRIPTION \n %s\n\n", grib_tool_description);
-    printf("\\section USAGE \n %s \n %s\n\n", grib_tool_name, grib_tool_usage);
+    printf("/*!  \\page %s %s\n", tool_name, tool_name);
+    printf("\\section DESCRIPTION \n %s\n\n", tool_description);
+    printf("\\section USAGE \n %s \n %s\n\n", tool_name, tool_usage);
     printf("\\section OPTIONS \n");
     for (i = 0; i < grib_options_count; i++) {
         if (grib_options[i].command_line) {
@@ -477,9 +481,9 @@ void usage_doxygen(void)
 #if 0
 void usage_doxygen(void) {
     int i=0;
-    printf("/*!  \\page %s %s\n",grib_tool_name,grib_tool_name);
-    printf("\\section DESCRIPTION \n%s\n\n",grib_tool_description);
-    printf("\\section USAGE \n%s \n%s\n\n",grib_tool_name,grib_tool_usage);
+    printf("/*!  \\page %s %s\n",tool_name,tool_name);
+    printf("\\section DESCRIPTION \n%s\n\n",tool_description);
+    printf("\\section USAGE \n%s \n%s\n\n",tool_name,tool_usage);
     printf("\\section OPTIONS\n");
     printf("<table frame=void border=0>\n");
     for (i=0;i<grib_options_count;i++) {
