@@ -60,7 +60,7 @@ GRIB_INLINE static int grib_inline_strcmp(const char* a, const char* b)
     return (*a == 0 && *b == 0) ? 0 : 1;
 }
 
-typedef double (*compare_double_proc)(double*, double*, double*);
+typedef double (*compare_double_proc)(const double*, const double*, const double*);
 
 typedef struct grib_error grib_error;
 struct grib_error
@@ -89,7 +89,7 @@ static int morein1             = 0;
 static int morein2             = 0;
 static int listFromCommandLine;
 static int verbose                = 0;
-static int tolerance_factor       = 1;
+static double tolerance_factor    = 1;
 static int write_error            = 0;
 static grib_handle* global_handle = NULL;
 static int global_counter         = 0;
@@ -97,7 +97,7 @@ static int start                  = -1;
 static int end                    = -1;
 static int write_count            = 0;
 
-GRIB_INLINE static double compare_double_absolute(double* a, double* b, double* err)
+GRIB_INLINE static double compare_double_absolute(const double* a, const double* b, const double* err)
 {
     double ret = 0;
     double d   = fabs(*a - *b);
@@ -144,7 +144,7 @@ static void write_messages(grib_handle* h1, grib_handle* h2)
     write_message(h2, "error2");
 }
 
-static double compare_double_relative(double* a, double* b, double* err)
+static double compare_double_relative(const double* a, const double* b, const double* err)
 {
     double relativeError;
 
@@ -258,7 +258,7 @@ int grib_tool_init(grib_runtime_options* options)
     }
 
     if (grib_options_on("r")) {
-        char* filename[1];
+        const char* filename[1];
         filename[0]      = options->infile_extra->name;
         options->random  = 1;
         options->orderby = strdup(orderby);
